package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.io.File;

/** action class for handling applets */
public class AppletAction extends Action {
    

    public Element process (Element message) {
	

	// find the stylesheet
	String stylesheet = GSFile.stylesheetFile(config_, "applet.xsl");

	if (stylesheet==null) {
	    System.err.println("AppletAction Error: applet stylesheet not found!");
	    return null;
	}
	
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	
	
	// get the collection and service param
	Element cgi_param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_param_list, false);

	// request_type is display (d) or request (r)
	String request_type = (String)params.get(GSCGI.REQUEST_TYPE_ARG);
	if (!request_type.equals("d")&&!request_type.equals("r")) {
	    System.err.println("AppletAction Error: the rt arg should be either d or r, instead it was "+request_type+"!");
	    return null;
	}

	String collection = (String)params.get(GSCGI.COLLECTION_ARG);
	String service_name=(String)params.get(GSCGI.SERVICE_ARG);
	
	String to=null;
	if (collection==null||collection.equals("") ) {
	    to = service_name;
	} else {
	    to = GSPath.appendLink(collection, service_name);
	}

	if (request_type.equals("d")) {
	    // we are just displaying the applet - get the description from
	    // the service, and process using xslt

	    //build up the mr request
	
	    Element mr_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    Element mr_request = doc_.createElement(GSXML.REQUEST_ELEM);
	    mr_message.appendChild(mr_request);
	    mr_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);
	    mr_request.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	    mr_request.setAttribute(GSXML.TO_ATT, to);
	    
	    Element mr_response = (Element)mr_.process(mr_message);
	    
	    // create the return page tree
	    Element page = doc_.createElement(GSXML.PAGE_ELEM);
	    page.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	    // add the lang stuff from message
	    page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.DISPLAY_ELEM), true));
	    // add the config stuff from message
	    page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));
	    
	    page.appendChild(doc_.importNode(request, true));

	    // add in the applet info
	    String path = GSXML.RESPONSE_ELEM;
	    path = GSPath.appendLink(path, GSXML.SERVICE_ELEM);
	    path = GSPath.appendLink(path, GSXML.APPLET_ELEM);
	    Element app_desc = (Element)doc_.importNode(GSXML.getNodeByPath(mr_response, path), true);
	    // must handle any params that have values that are not 
	    // necessarily known by the service
	    editLocalParams(app_desc, config_.library_name_, collection);
	    page.appendChild(app_desc);

	    // process using the stylesheet
	    Document style_doc = converter_.getDOM(new File(stylesheet));
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return (Element)transformer_.transform(style_doc, page);    

	}
	else if (request_type.equals("r")) {
	    // we are processing stuff for the applet send a message to the service, type="query", and take out the something element, and return that as our result - the applet must take xml

	    Element mr_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    Element mr_request = doc_.createElement(GSXML.REQUEST_ELEM);
	    mr_message.appendChild(mr_request);
	    mr_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);
	    mr_request.setAttribute(GSXML.TO_ATT, to);
	    mr_request.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));

	    // just append the params for now
	    mr_request.appendChild(doc_.importNode(cgi_param_list, true));
	    
	    Element mr_response = (Element)mr_.process(mr_message);
	    // add in the applet data
	    String path = GSXML.RESPONSE_ELEM;
	    path = GSPath.appendLink(path, GSXML.APPLET_DATA_ELEM);
	    Element applet_info = (Element)GSXML.getNodeByPath(mr_response, path).getFirstChild();
	    return applet_info;

	}
	// should never get here
	return null;
    }

    /** this method looks through the PARAMs of the applet description for 
     * one with the name 'library'. If its found, it sets the value to be 
     * library_name
     */
    protected void editLocalParams(Element description, String library_name,
				   String full_collection_name) {

	Node child = description.getFirstChild();
	while (child != null) {
	    if (child.getNodeType() == Node.ELEMENT_NODE) {
		String param = child.getNodeName();
		if (param.equals("PARAM")||param.equals("param")) {
		    String name = ((Element)child).getAttribute("NAME");
		    if (name == null || name.equals("")) {
			// somethings wrong!!
		    } else if (name.equals("library")) {
			((Element)child).setAttribute("VALUE", library_name);
		    } else if (name.equals("collection")) {
			((Element)child).setAttribute("VALUE", full_collection_name);
		    }
		    
		}
	    }
	    child = child.getNextSibling();
	}
    }
	
}
