package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Text; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.io.File;

/** action class for queries */
public class QueryAction extends Action {
    
    /** process - processes a request.
     */ 
    public Element process (Element message) {

	// get the request - assume there is only one
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);

	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.DISPLAY_ELEM), true));
	// add the system stuff from message
       page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

       // if want to have a different type of query here, check the subaction att of request

       // for now assume all queries can be handled by basic query
       return basicQuery(page, request);
       
    }
    
    /** a generic query handler 
     * this gets the service description, does the query (just passes all teh 
     * params to the service, then gets the titles for any results 
     */
    protected Element basicQuery(Element page, Element request) {

	// check that the stylesheet is present - cant output the page without one. 
	String stylesheet = GSFile.stylesheetFile(config_, "basicquery.xsl");
	if (stylesheet==null) {
	    System.err.println("QueryAction Error: basicquery stylesheet not found!");
	    return null;
	}
	Document style_doc = converter_.getDOM(new File(stylesheet));

	// part of the data for the page is the cgi-params
	// if we have this here, do we need to overwrite default values in the 
	// param list down below??
	Element cgi_request = (Element)doc_.importNode(request, true);
	page.appendChild(cgi_request);

	// extract the params from the cgi-request, and check that we have a coll specified
	Element cgi_param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_param_list, false);
	
	String request_type = (String)params.get(GSCGI.REQUEST_TYPE_ARG);
	String service_name = (String)params.get(GSCGI.SERVICE_ARG);
	String collection = (String)params.get(GSCGI.COLLECTION_ARG);
	if (collection == null || collection.equals("")) {
	    System.err.println("QueryAction Error: no collection was specified!");
	    return null; 
	}
	
	String lang = request.getAttribute(GSXML.LANG_ATT);
	String to = GSPath.appendLink(collection, service_name);

	// the second part of the page is the service description 
	// for now get this again from the service. 
	// this will probably need to be cached somehow later on. 
	Element mr_info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_info_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE,  to, lang);
	mr_info_message.appendChild(mr_info_request);

	// also get the format stuff now if there is some
	Element format_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_FORMAT, to, lang);
	mr_info_message.appendChild(format_request);

	Element mr_info_response = (Element) mr_.process(mr_info_message);
	
	String path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.SERVICE_ELEM);
	Element description = (Element)doc_.importNode(GSXML.getNodeByPath(mr_info_response, path), true);
	Element pl = (Element)GSXML.getChildByTagName(description, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	
	if (pl !=null) {
	    // add short names to the params in the param list
	    cgi_.paramListAddShortNames(pl);
	    // for each param in the description, overwrite teh default value with the currently set value if present 
	    Element param = (Element)pl.getFirstChild();
	    while (param !=null) {
		if (param.getNodeName().equals(GSXML.PARAM_ELEM)) { // just in case
		    if (param.getAttribute(GSXML.TYPE_ATT).equals(GSXML.PARAM_TYPE_MULTI)) {
			// get the values for each sub param
			NodeList subparams = param.getElementsByTagName(GSXML.PARAM_ELEM);
			for (int i=0; i<subparams.getLength(); i++) {
			    String name = ((Element)subparams.item(i)).getAttribute(GSXML.NAME_ATT);
			    String current = (String)params.get(name);
			    if (current !=null && !current.equals("")) {
				Element e = GSXML.createTextElement(pl.getOwnerDocument(), GSXML.DEFAULT_ELEM, current);
				e.setAttribute(GSXML.NAME_ATT, name);
				param.appendChild(e);
			    }
			}
		    } else {
			
			String name = param.getAttribute(GSXML.NAME_ATT);
			String current = (String)params.get(name);
			if (current !=null && !current.equals("")) {
			    param.setAttribute(GSXML.DEFAULT_ATT, current);
			}
		    }
		}
		param = (Element)param.getNextSibling();
	    }
	}
    
	// append the description to the page
	page.appendChild(description);
	
	Element format_response = (Element)mr_info_response.getElementsByTagName(GSXML.RESPONSE_ELEM).item(1); // the second response

	// just a display request
	if (request_type.equals("d")) {
	    // output the page
	    // process using the stylesheet
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return (Element)transformer_.transform(style_doc, page);    
	}
	
	// do the query
	Element mr_query_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_query_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_PROCESS, to, page.getAttribute(GSXML.LANG_ATT));
	mr_query_message.appendChild(mr_query_request);
	
	// paramList
	Element query_param_list = (Element)doc_.importNode(cgi_param_list, true);
	mr_query_request.appendChild(query_param_list);
	    
	Element mr_query_response = (Element)mr_.process(mr_query_message);

	path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	Element query_result_metadata_list = (Element) GSXML.getNodeByPath(mr_query_response, path);
	if (query_result_metadata_list == null) {
	    System.err.println("QueryAction: Warning: No query result metadata.\n");
	}
	path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.TERM_ELEM+GSXML.LIST_MODIFIER);
	Element query_term_info_list = (Element) GSXML.getNodeByPath(mr_query_response, path);
	if (query_term_info_list == null) {
	    System.err.println("QueryAction: Warning: No query term information.\n");
	}

	// this result is the list of docs.
	// now take that list, and get the Titles
	// for now, just create a whole new request
	
	// check that there are some documents - for now check the list, but later should use a numdocs metadata elem
	path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	
	Element document_list = (Element)GSXML.getNodeByPath(mr_query_response,
							     path); 
	// documentList not present if no docs found
	if (document_list == null) {
	    
	    Element result_response = (Element)GSXML.getChildByTagName(mr_query_response, GSXML.RESPONSE_ELEM);
	    
	    page.appendChild(doc_.importNode(result_response, true));
	    
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return (Element)transformer_.transform(style_doc, page);    
	    
	}
	
	// add in the format info
	Element format_elem = getAndTransformFormat(format_response);
	if (format_elem != null) {
	    
	    GSXSLT.mergeStylesheets(style_doc, format_elem);
	}
	

	// we have a doc list, so get the metadata - for now, get title.
	// can we dynamically decide what metadata to get?
	Element mr_metadata_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_metadata_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_metadata_message.appendChild(mr_metadata_request);
	
	mr_metadata_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);
	mr_metadata_request.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));
	to = GSPath.appendLink(collection, "DocumentMetadataRetrieve"); // Hard-wired?
	mr_metadata_request.setAttribute(GSXML.TO_ATT, to);

	// Create a parameter list to specify the required metadata information
	Element dm_param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	mr_metadata_request.appendChild(dm_param_list);
	Element dm_param = doc_.createElement(GSXML.PARAM_ELEM);
	dm_param_list.appendChild(dm_param);
	dm_param.setAttribute(GSXML.NAME_ATT, "metadata");
	dm_param.setAttribute(GSXML.VALUE_ATT, "Title");

	mr_metadata_request.appendChild(doc_.importNode(document_list, true));
	Element mr_metadata_response = (Element)mr_.process(mr_metadata_message);	
	Element result_response = (Element)GSXML.getChildByTagName(mr_metadata_response, GSXML.RESPONSE_ELEM);
	
	// If there is some metadata about the query results, add it in
	if (query_result_metadata_list != null) {
	    result_response.appendChild(query_result_metadata_list);
	}
	// If there is some metadata about the query results, add it in
	if (query_term_info_list != null) {
	    result_response.appendChild(query_term_info_list);
	}

	page.appendChild(doc_.importNode(result_response, true));
	
	// output the page
	// process using the stylesheet
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return (Element)transformer_.transform(style_doc, page);
	
    }
}
