/*
 *    Collection.java
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.collection;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.core.*;
import org.greenstone.gsdl3.service.*;


// java XML classes we're using
import org.w3c.dom.Document; 
import org.w3c.dom.Node; 
import org.w3c.dom.Element; 
import org.w3c.dom.NodeList; 

import java.io.File;
import java.util.HashMap;

/**
 * Represents a collection in Greenstone. A collection is an extension of
 * a ServiceCluster - it has local data that the services use.
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 3934 $
 * @see ModuleInterface
 */
public class Collection 
    extends ServiceCluster {

    protected XMLTransformer transformer_ = null;
    /** same as setClusterName */
    public void setCollectionName(String name) {
	setClusterName(name);
    }

    public Collection() {
	service_map_ = new HashMap();
	converter_ = new XMLConverter();
	transformer_ = new XMLTransformer();
	doc_ = converter_.newDOM();
	description_ = doc_.createElement(GSXML.COLLECTION_ELEM);
	service_info_ = doc_.createElement(GSXML.SERVICE_ELEM+GSXML.LIST_MODIFIER);
	meta_info_ = doc_.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	description_.appendChild(meta_info_);
	description_.appendChild(service_info_);
	
    }
	
    /**
     * Configures the collection.
     *
     * gsdlHome and collectionName must be set before configure is called.
     *
     * the file buildcfg.xml is located in gsdlHome/collect/collectionName
     * collection metadata is obtained, and services loaded.
     *
     * @return true/false on success/fail
     */
    public boolean configure() {
	
	if (site_home_ == null || cluster_name_== null) {
	    System.err.println("site_home and collection_name must be set before configure called!");
	    return false;
	}
	// read the collection build configuration file
	File build_config_file = new File(GSFile.collectionBuildConfigFile(site_home_, cluster_name_));
	File coll_config_file = new File(GSFile.collectionConfigFile(site_home_, cluster_name_));

	if (!coll_config_file.exists()) {
	    System.err.println(coll_config_file+" does not exist");
	    System.err.println("couldn't configure collection: "+cluster_name_);
	    return false;
	}
	
	if (!build_config_file.exists()) {
	    System.err.println(build_config_file+" does not exist");
	    System.err.println("couldn't configure collection: "+cluster_name_);
	    return false;
	}
	
	// get the xml for both files
	Element coll_config_xml = converter_.getDOM(coll_config_file).getDocumentElement();
	Element build_config_xml = converter_.getDOM(build_config_file).getDocumentElement();
	
	// process the metadata
	Element meta_list = (Element)GSXML.getChildByTagName(coll_config_xml, GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	configureMetadata(meta_list);
	meta_list = (Element)GSXML.getChildByTagName(build_config_xml, GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	configureMetadata(meta_list);
	
	// now do the services
	
	Element service_list = (Element)GSXML.getChildByTagName(build_config_xml, GSXML.SERVICE_CLASS_ELEM+GSXML.LIST_MODIFIER);
	configureServiceRack(service_list, coll_config_xml);
	
	return true;
	
    }

}




